@@ -134,7 +134,7 @@ ClientProto Client::serialize(unordered_map<const string *, size_t> *short_respo
 		serialized.set_header_or_short_response_index(iterator_and_inserted.first->second);
 	}
 
-	serialized.set_header_or_short_response_bytes_sent(serialized.header_or_short_response_bytes_sent());
+	serialized.set_header_or_short_response_bytes_sent(header_or_short_response_bytes_sent);
 	serialized.set_stream_pos(stream_pos);
 	serialized.set_stream_pos_end(stream_pos_end);
 	serialized.set_bytes_sent(bytes_sent);
