@@ -167,6 +167,7 @@ int HTTPInput::lookup_and_connect(const string &host, const string &port)
 		if (ioctl(sock, FIONBIO, &one) == -1) {
 			log_perror("ioctl(FIONBIO)");
 			safe_close(sock);
+			freeaddrinfo(base_ai);
 			return -1;			
 		}
 
@@ -339,7 +340,8 @@ void HTTPInput::do_work()
 			pending_data.clear();
 			has_metacube_header = false;
 			for (int stream_index : stream_indices) {
-				servers->set_header(stream_index, "", "");
+				// Don't zero out the header; it might still be of use to HLS clients.
+				servers->set_unavailable(stream_index);
 			}
 
 			{
