@@ -98,23 +98,23 @@ Input *create_input(const string &url, Input::Encoding encoding)
 {
 	string protocol, user, host, port, path;
 	if (!parse_url(url, &protocol, &user, &host, &port, &path)) {
-		return NULL;
+		return nullptr;
 	}
 	if (protocol == "http") {
 		return new HTTPInput(url, encoding);
 	}
 	if (protocol == "udp") {
-		// encoding is ignored; it's never Metacube.
+		assert(encoding == Input::INPUT_ENCODING_RAW);
 		return new UDPInput(url);
 	}
-	return NULL;
+	return nullptr;
 }
 
 Input *create_input(const InputProto &serialized)
 {
 	string protocol, user, host, port, path;
 	if (!parse_url(serialized.url(), &protocol, &user, &host, &port, &path)) {
-		return NULL;
+		return nullptr;
 	}
 	if (protocol == "http") {
 		return new HTTPInput(serialized);
@@ -122,7 +122,7 @@ Input *create_input(const InputProto &serialized)
 	if (protocol == "udp") {
 		return new UDPInput(serialized);
 	}
-	return NULL;
+	return nullptr;
 }
 
 Input::~Input() {}
