@@ -1,6 +1,7 @@
 #ifndef _INPUT_H
 #define _INPUT_H 1
 
+#include <math.h>
 #include <stddef.h>
 #include <time.h>
 #include <string>
@@ -17,23 +18,23 @@ struct InputStats {
 	// The number of bytes we have received so far, including any Metacube headers.
 	//
 	// Not reset across connections.
-	size_t bytes_received;
+	size_t bytes_received = 0;
 
 	// The number of data bytes we have received so far (or more precisely,
 	// number of data bytes we have sent on to the stream). This excludes Metacube
 	// headers, metadata and corrupted data we've skipped.
 	//
 	// Not reset across connections.
-	size_t data_bytes_received;
+	size_t data_bytes_received = 0;
 
 	// Same, except counts only Metacube metadata.
-	size_t metadata_bytes_received;
+	size_t metadata_bytes_received = 0;
 
 	// When the current connection was initiated. -1 if we are not currently connected.
-	time_t connect_time;
+	time_t connect_time = -1;
 
 	// Last latency measurement, HUGE_VAL if no measurement yet.
-	double latency_sec;
+	double latency_sec = HUGE_VAL;
 
 	// TODO: Number of loss events might both be useful,
 	// similar to for clients. Also, per-connection byte counters.
@@ -58,7 +59,7 @@ public:
 bool parse_url(const std::string &url, std::string *protocol, std::string *user, std::string *host, std::string *port, std::string *path);
 
 // Figure out the right type of input based on the URL, and create a new Input of the right type.
-// Will return NULL if unknown.
+// Will return nullptr if unknown.
 Input *create_input(const std::string &url, Input::Encoding encoding);
 Input *create_input(const InputProto &serialized);
 
