@@ -39,7 +39,7 @@ void InputStatsThread::do_work()
 		}
 
 		fp = fdopen(fd, "w");
-		if (fp == NULL) {
+		if (fp == nullptr) {
 			log_perror("fdopen");
 			safe_close(fd);
 			if (unlink(filename) == -1) {
@@ -49,7 +49,7 @@ void InputStatsThread::do_work()
 			goto sleep;
 		}
 
-		now = time(NULL);
+		now = time(nullptr);
 		for (size_t i = 0; i < inputs.size(); ++i) {
 			InputStats stats = inputs[i]->get_stats();
 			fprintf(fp, "%s %llu %llu", stats.url.c_str(),
