@@ -56,9 +56,9 @@ vector<string> split_lines(const string &str)
 	return ret;
 }
 
-multimap<string, string> extract_headers(const vector<string> &lines, const string &log_context)
+HTTPHeaderMultimap extract_headers(const vector<string> &lines, const string &log_context)
 {
-	multimap<string, string> parameters;
+	HTTPHeaderMultimap parameters;
 	for (size_t i = 1; i < lines.size(); ++i) {
 		size_t split = lines[i].find(":");
 		if (split == string::npos) {
@@ -100,7 +100,7 @@ RequestParseStatus wait_for_double_newline(string *existing_data, const char *ne
 	const char *ptr = reinterpret_cast<char *>(
 		memmem(existing_data->data() + start_at, existing_data->size() - start_at,
 		       "\r\n\r\n", 4));
-	if (ptr == NULL) {
+	if (ptr == nullptr) {
 		return RP_NOT_FINISHED_YET;
 	}
 	if (ptr != existing_data->data() + existing_data->size() - 4) {
