@@ -178,6 +178,15 @@ void ServerPool::add_data(int stream_index, const char *data, size_t bytes, uint
 	}
 }
 
+void ServerPool::set_unavailable(int stream_index)
+{
+	assert(stream_index >= 0 && stream_index < ssize_t(num_http_streams));
+
+	for (int i = 0; i < num_servers; ++i) {
+		servers[i].set_unavailable(stream_index);
+	}
+}
+
 void ServerPool::add_gen204(const std::string &url, const std::string &allow_origin)
 {
 	for (int i = 0; i < num_servers; ++i) {
