@@ -210,7 +210,7 @@ void UDPInput::do_work()
 			ret = recv(sock, packet_buf, sizeof(packet_buf), 0);
 		} while (ret == -1 && errno == EINTR);
 
-		if (ret <= 0) {
+		if (ret < 0) {  // Note that zero-byte packets are legal.
 			log_perror("recv");
 			close_socket();
 			continue;
