@@ -16,7 +16,7 @@
 
 using namespace std;
 
-int make_tempfile(const std::string &contents)
+int make_tempfile(const string &contents)
 {
 	int fd = open("/tmp", O_RDWR | O_TMPFILE, 0600);
 	if (fd == -1) {
@@ -52,14 +52,14 @@ int make_tempfile(const std::string &contents)
 	return fd;
 }
 
-bool read_tempfile_and_close(int fd, std::string *contents)
+bool read_tempfile_and_close(int fd, string *contents)
 {
 	bool ok = read_tempfile(fd, contents);
 	safe_close(fd);  // Implicitly deletes the file.
 	return ok;
 }
 
-bool read_tempfile(int fd, std::string *contents)
+bool read_tempfile(int fd, string *contents)
 {
 	ssize_t ret, has_read;
 
